pub fn multiply(comptime InstrFn: type, comptime L: bool, comptime U: bool, comptime A: bool, comptime S: bool) InstrFn {
    const Arm32 = @typeInfo(@typeInfo(@typeInfo(InstrFn).Pointer.child).Fn.params[0].type.?).Pointer.child;

    return struct {
        fn inner(cpu: *Arm32, opcode: u32) void {
            const rd = opcode >> 16 & 0xF;
            const rn = opcode >> 12 & 0xF;
            const rs = opcode >> 8 & 0xF;
            const rm = opcode & 0xF;

            if (L) {
                const rd_hi = rd;
                const rd_lo = rn;

                if (U) {
                    // Signed (WHY IS IT U THEN?)
                    var result: i64 = @as(i64, @as(i32, @bitCast(cpu.r[rm]))) * @as(i64, @as(i32, @bitCast(cpu.r[rs])));
                    if (A) result +%= @bitCast(@as(u64, cpu.r[rd_hi]) << 32 | @as(u64, cpu.r[rd_lo]));

                    cpu.r[rd_hi] = @bitCast(@as(i32, @truncate(result >> 32)));
                    cpu.r[rd_lo] = @bitCast(@as(i32, @truncate(result)));
                } else {
                    // Unsigned
                    var result: u64 = @as(u64, cpu.r[rm]) * @as(u64, cpu.r[rs]);
                    if (A) result +%= @as(u64, cpu.r[rd_hi]) << 32 | @as(u64, cpu.r[rd_lo]);

                    cpu.r[rd_hi] = @truncate(result >> 32);
                    cpu.r[rd_lo] = @truncate(result);
                }

                if (S) {
                    cpu.cpsr.z.write(cpu.r[rd_hi] == 0 and cpu.r[rd_lo] == 0);
                    cpu.cpsr.n.write(cpu.r[rd_hi] >> 31 & 1 == 1);
                    // C and V are set to meaningless values
                }
            } else {
                const temp: u64 = @as(u64, cpu.r[rm]) * @as(u64, cpu.r[rs]) + if (A) cpu.r[rn] else 0;
                const result: u32 = @truncate(temp);
                cpu.r[rd] = result;

                if (S) {
                    cpu.cpsr.n.write(result >> 31 & 1 == 1);
                    cpu.cpsr.z.write(result == 0);
                    // V is unaffected, C is *actually* undefined in ARMv4
                }
            }
        }
    }.inner;
}
